				  require "libs.core"
local KBIO		= require "libs.kbio"



-- =========================================================================
-- {[> Attributes
-- ==============  {{{
local flipMap = { --{{{
	["h"] = {
		{ --{{{ curved-rail
			[1] = {
				["curved-rail"] = true
			},
			[2] = {
				[1] = {0, 0, 1},
				[2] = {0, 0, 0},
				[3] = {0, 0, 7},
				[4] = {0, 0, 6},
				[5] = {0, 0, 5},
				[6] = {0, 0, 4},
				[7] = {0, 0, 3},
				[8] = {0, 0, 2}
			},
			[3] = nil
		}, --}}}
		{ --{{{ rail-signal, rail-chain-signal
			[1] = {
				["rail-signal"] = true,
				["rail-chain-signal"] = true
			},
			[2] = {
				[1] = { 3,  0, 0},
				[2] = { 2,  2, 7},
				[3] = { 0,  3, 6},
				[4] = {-2,  2, 5},
				[5] = {-3,  0, 4},
				[6] = {-2, -2, 3},
				[7] = { 0, -3, 2},
				[8] = { 2, -2, 1}
			},
			[3] = nil
		}, --}}}
		{ --{{{ train-stop
			[1] = {
				["train-stop"] = true
			},
			[2] = {
				[1] = {-4,  0, 0},
				[3] = { 0, -4, 6},
				[5] = { 4,  0, 4},
				[7] = { 0,  4, 2}
			},
			[3] = nil
		}, --}}}
		{ --{{{ storage-tank
			[1] = {
				["storage-tank"] = true
			},
			[2] = {
				[1] = {0, 0, 2},
				[2] = {0, 0, 2},
				[3] = {0, 0, 4},
				[4] = {0, 0, 2},
				[5] = {0, 0, 2},
				[6] = {0, 0, 2},
				[7] = {0, 0, 4},
				[8] = {0, 0, 2}
			},
			[3] = nil
		}, --}}}
		{ --{{{ y_inserter_smart_LL
			[1] = {
				["y_inserter_smart_LL"] = true
			},
			[2] = {
				[1] = {0, 0, 0},
				[3] = {0, 0, 7},
				[5] = {0, 0, 4},
				[7] = {0, 0, 2}
			},
			[3] = "y_inserter_smart_RR",
		}, --}}}
		{ --{{{ y_inserter_smart_RR
			[1] = {
				["y_inserter_smart_RR"] = true
			},
			[2] = {
				[1] = {0, 0, 0},
				[3] = {0, 0, 7},
				[5] = {0, 0, 4},
				[7] = {0, 0, 2}
			},
			[3] = "y_inserter_smart_LL"
		}, --}}}
		{ --{{{ y_inserter_smart_leftR2
			[1] = {
				["y_inserter_smart_leftR2"] = true
			},
			[2] = {
				[1] = {0, 0, 0},
				[3] = {0, 0, 7},
				[5] = {0, 0, 4},
				[7] = {0, 0, 2}
			},
			[3] = "y_inserter_smart_rightR2",
		}, --}}}
		{ --{{{ y_inserter_smart_rightR2
			[1] = {
				["y_inserter_smart_rightR2"] = true
			},
			[2] = {
				[1] = {0, 0, 0},
				[3] = {0, 0, 7},
				[5] = {0, 0, 4},
				[7] = {0, 0, 2}
			},
			[3] = "y_inserter_smart_leftR2"
		}, --}}}
	},
	["v"] = {
		{ --{{{ curved-rail
			[1] = {
				["curved-rail"] = true
			},
			[2] = {
				[1] = {0, 0, 5},
				[2] = {0, 0, 4},
				[3] = {0, 0, 3},
				[4] = {0, 0, 2},
				[5] = {0, 0, 1},
				[6] = {0, 0, 0},
				[7] = {0, 0, 7},
				[8] = {0, 0, 6}
			},
			[3] = nil
		}, --}}}
		{ --{{{ rail-signal, rail-chain-signal
			[1] = {
				["rail-signal"] = true,
				["rail-chain-signal"] = true
			},
			[2] = {
				[1] = { 3,  0, 4},
				[2] = { 2,  2, 3},
				[3] = { 0,  3, 2},
				[4] = {-2,  2, 1},
				[5] = {-3,  0, 0},
				[6] = {-2, -2, 7},
				[7] = { 0, -3, 6},
				[8] = { 2, -2, 5}
			},
			[3] = nil
		}, --}}}
		{ --{{{ train-stop
			[1] = {
				["train-stop"] = true
			},
			[2] = {
				[1] = {-4,  0, 4},
				[3] = { 0, -4, 2},
				[5] = { 4,  0, 0},
				[7] = { 0,  4, 6}
			},
			[3] = nil
		}, --}}}
		{ --{{{ storage-tank
			[1] = {
				["storage-tank"] = true
			},
			[2] = {
				[1] = {0, 0, 2},
				[2] = {0, 0, 2},
				[3] = {0, 0, 4},
				[4] = {0, 0, 2},
				[5] = {0, 0, 2},
				[6] = {0, 0, 2},
				[7] = {0, 0, 4},
				[8] = {0, 0, 2}
			},
			[3] = nil
		}, --}}}
		{ --{{{ y_inserter_smart_LL
			[1] = {
				["y_inserter_smart_LL"] = true
			},
			[2] = {
				[1] = {0, 0, 4},
				[3] = {0, 0, 2},
				[5] = {0, 0, 1},
				[7] = {0, 0, 6}
			},
			[3] = "y_inserter_smart_RR"
		}, --}}}
		{ --{{{ y_inserter_smart_RR
			[1] = {
				["y_inserter_smart_RR"] = true
			},
			[2] = {
				[1] = {0, 0, 4},
				[3] = {0, 0, 2},
				[5] = {0, 0, 1},
				[7] = {0, 0, 6}
			},
			[3] = "y_inserter_smart_LL"
		}, --}}}
		{ --{{{ y_inserter_smart_leftR2
			[1] = {
				["y_inserter_smart_leftR2"] = true
			},
			[2] = {
				[1] = {0, 0, 4},
				[3] = {0, 0, 2},
				[5] = {0, 0, 1},
				[7] = {0, 0, 6}
			},
			[3] = "y_inserter_smart_rightR2",
		}, --}}}
		{ --{{{ y_inserter_smart_rightR2
			[1] = {
				["y_inserter_smart_rightR2"] = true
			},
			[2] = {
				[1] = {0, 0, 4},
				[3] = {0, 0, 2},
				[5] = {0, 0, 1},
				[7] = {0, 0, 6}
			},
			[3] = "y_inserter_smart_leftR2"
		}, --}}}
	}
} --}}}
--}}}



-- =========================================================================
-- {[> Methods
-- ===========
function addEntry(player, data) --{{{

	table.insert(global[player.name].database, data)
	sortBlueprints(player)
	
end --}}}

function checkEntryName(player, name) --{{{

	for _, t in ipairs(global[player.name].database) do
		if (t.data.label:lower() == name:lower()) then
			player.print({"msg_error_duplicate_entry", name})
			return false
		end
	end
	
	return true

end --}}}

function createOrImportEntry(player) --{{{

	local string = player.gui.center["kblueprints_popup"]["kblueprints_popupInputField"].text:trim()

	if global[player.name].tmpBlueprintData then

		if (not checkEntryName(player, string)) then
			return false
		end
		
		global[player.name].tmpBlueprintData.data.label = string
		
		addEntry(player, global[player.name].tmpBlueprintData)

	else

		if (string:len() == 0) then
			player.print({"msg_error_no_string"})
			return false
		end
	
		local data, structure = KBIO.fromString(string)
		
		if (not data) then
			player.print({"msg_error_unable_to_parse"})
			return
		end
		
		if (structure == "collection") then
			player.print({"msg_error_collection_to_entry"})
			return
		end
		
		if (not data.data.label) then
			data.data.label = generateNewName(player)
		elseif (not checkEntryName(player, data.data.label)) then
			return false
		end
		
		addEntry(player, data)

	end

	player.gui.center["kblueprints_popup"].destroy()
	return true

end --}}}

function deleteEntry(player, index) --{{{

	table.remove(global[player.name].database, index)
	
	if player.gui.center["kblueprints_popup"] then
		player.gui.center["kblueprints_popup"].destroy()
	end
	
	return true

end --}}}

function exportCollection(player) --{{{

	local data = global[player.name].database

	if data and (#(data) > 0) then
	
		local string = KBIO.toString(global[player.name].database)
	
		if string then
		
			local fileName = player.name
			
			fileName = fileName:gsub("<>?\":|\\/*", "_") -- "
			
			game.write_file("KBlueprints/collection/" .. fileName .. ".txt", string, false)
			player.print({"msg_collection_exported"})
		
		else
		
			player.print({"msg_error_unable_to_parse"})
			
		end
	
	end
	
end --}}}

function exportEntry(player, index) --{{{

	local data = global[player.name].database[index]
	local fileName = player.name .. " - " .. data.data.label
	
	fileName = fileName:gsub("<>?\":|\\/*", "_") -- "
	
	game.write_file("KBlueprints/" .. fileName .. ".txt", KBIO.toString(data), false)
	player.print({"msg_entry_exported", fileName})
	
end --}}}

function flip(player, type) --{{{

	local stack = getBlueprintStackFromCursor(player)
	
	if stack then
	
		local showWarning
		local foundEntity
		local direction
	
		-- entities
		local entities = stack.get_blueprint_entities()
		
		if entities then
		
			for _, entity in ipairs(entities) do
			
				if (entity.name == "rail-signal") or (entity.name == "rail-chain-signal") or (entity.name == "train-stop") then
					showWarning = true
				end
			
				direction = entity.direction or 0
				foundEntity = false
				
				for _, t in ipairs(flipMap[type]) do
				
					if t[1][entity.name] then
					
						entity.position.x = entity.position.x + t[2][direction + 1][1]
						entity.position.y = entity.position.y + t[2][direction + 1][2]
						
						entity.direction = t[2][direction + 1][3]
						
						if t[3] then
							entity.name = t[3]
						end
					
						foundEntity = true
						
						break
						
					end
				end
				
				if (type == "h") then
					if (not foundEntity) then
						entity.direction = (16 - direction) % 8
					end
					entity.position.x = -entity.position.x
				else
					if (not foundEntity) then
						entity.direction = (12 - direction) % 8
					end
					entity.position.y = -entity.position.y
				end
				
			end
		
			stack.set_blueprint_entities(entities)
			
		end
		
		
		
		-- tiles
		local tiles = stack.get_blueprint_tiles()
		
		if tiles then
		
			for _, tile in ipairs(tiles) do
			
				direction = tile.direction or 0
				
				if (type == "h") then
					tile.direction = (16 - direction) % 8
					tile.position.x = -tile.position.x
				else
					tile.direction = (12 - direction) % 8
					tile.position.y = -tile.position.y
				end
				
			end
		
			stack.set_blueprint_tiles(tiles)
		
		end
		
		
		
		-- show warning if rail signals and/or train stops were flipped
		if showWarning then
			player.print({"msg_warning_flipped"})
		end
	
	end

end --}}}

function flipHorizontally(player) --{{{
	flip(player, "h")
end --}}}

function flipVertically(player) --{{{
	flip(player, "v")
end --}}}

function generateNewName(player) --{{{
	
	local names = {}
	for _, t in ipairs(global[player.name].database) do
		names[t.data.label] = true
	end
	
	local i = 1
	local name

	while true do
	
		name = "_newEntry" .. i
		
		if (not names[name]) then
			return name
		end
		
		i = i + 1
	
	end
	
	return name

end --}}}

function getBlueprintStackFromCursor(player) --{{{
	
	local stack = player.cursor_stack
	
	if stack.valid_for_read then
	
		if (stack.name == "blueprint") then
		
			if stack.is_blueprint_setup() then
				return stack
			end
			
		elseif (stack.name == "blueprint-book") then
			
			local inv = stack.get_inventory(defines.inventory.item_main)
			
			if (inv[1].valid_for_read and inv[1].name == "blueprint") then
			
				if inv[1].is_blueprint_setup() then
					return inv[1]
				end
				
			end
		end
	end
	
	return nil

end --}}}

function getDataTableFromCursor(player) --{{{

	local data = nil
	
	local stack = player.cursor_stack
	
	if stack.valid_for_read then
	
		if (stack.name == "blueprint") and stack.is_blueprint_setup() then
		
			data = {
				type = "blueprint",
				version = KBIO.getGameVersion(),
				data = {
					label = stack.label,
					entities = stack.get_blueprint_entities(),
					tiles = stack.get_blueprint_tiles(),
					icons = stack.blueprint_icons
				}
			}
			
		elseif (stack.name == "blueprint-book") then
			
			data = {
				type = "blueprint-book",
				version = KBIO.getGameVersion(),
				data = {
					label = stack.label
				}
			}
			
			local inventory = stack.get_inventory(defines.inventory.item_main)
			
			for i = 1, #(inventory) do
				if (inventory[i].valid_for_read and inventory[i].is_blueprint_setup()) then
				
					if (not data.data.main) then
						data.data.main = {}
					end
				
					table.insert(data.data.main, {
						label = inventory[i].label,
						entities = inventory[i].get_blueprint_entities(),
						tiles = inventory[i].get_blueprint_tiles(),
						icons = inventory[i].blueprint_icons
					})
				end
			end
			
		end
		
	end
	
	return data

end --}}}

function getWritableBlueprint(player) --{{{

	-- blueprint on cursor?
	local stack = player.cursor_stack
	if (stack.valid_for_read and (stack.name == "blueprint")) then
		return stack
	end
	
	-- empty blueprint on quickbar?
	local inv = player.get_inventory(defines.inventory.player_quickbar)
	for i = 1, #(inv) do
		if (inv[i].valid_for_read and (inv[i].name == "blueprint") and (not inv[i].is_blueprint_setup())) then
			return inv[i]
		end
	end
	
	-- empty blueprint in inventory?
	inv = player.get_inventory(defines.inventory.player_main)
	for i = 1, #(inv) do
		if (inv[i].valid_for_read and (inv[i].name == "blueprint") and (not inv[i].is_blueprint_setup())) then
			return inv[i]
		end
	end

end --}}}

function getWritableBlueprintBook(player) --{{{

	-- blueprint book on cursor?
	local stack = player.cursor_stack
	if (stack.valid_for_read and (stack.name == "blueprint-book")) then
		return stack
	end
	
	-- empty blueprint on quickbar?
	local inv = player.get_inventory(defines.inventory.player_quickbar)
	for i = 1, #(inv) do
		if (inv[i].valid_for_read and (inv[i].name == "blueprint-book") and isBlueprintBookEmpty(inv[i])) then
			return inv[i]
		end
	end
	
	-- empty blueprint in inventory?
	inv = player.get_inventory(defines.inventory.player_main)
	for i = 1, #(inv) do
		if (inv[i].valid_for_read and (inv[i].name == "blueprint-book") and isBlueprintBookEmpty(inv[i])) then
			return inv[i]
		end
	end

end --}}}

function importCollection(player) --{{{

	local string = player.gui.center["kblueprints_popup"]["kblueprints_popupInputField"].text:trim()

	if (string:len() == 0) then
		player.print({"msg_error_no_string"})
		return false
	end
	
	local data, structure = KBIO.fromString(string)
	
	if (not data) then
		player.print({"msg_error_unable_to_parse"})
		return
	end
	
	if (structure ~= "collection") then
		player.print({"msg_error_string_is_no_collection"})
		return
	end
	
	while (#(global[player.name].database) > 0) do
		table.remove(global[player.name].database, 1)
	end
	
	for i, entry in ipairs(data) do
		
		if (not entry.data.label) then
			data[i].data.label = generateNewName(player)
			addEntry(player, entry)
		else
			if checkEntryName(player, entry.data.label) then
				addEntry(player, entry)
			end
		end
	
	end
	
	player.gui.center["kblueprints_popup"].destroy()
	
	return true
	
end --}}}

function initializePlayer(player) --{{{

	if (not global[player.name]) then
		global[player.name] = {}
	end
	
	if (not global[player.name].database) then
		global[player.name].database = {}
	end
	
	createMainButton(player)

end --}}}

function isBlueprintBookEmpty(stack) --{{{

	local inv = stack.get_inventory(defines.inventory.item_main)
	for i = 1, #(inv) do
		if (inv[i].valid_for_read and inv[i].is_blueprint_setup()) then
			return false
		end
	end
	
	return true

end --}}}

function loadEntry(player, index) --{{{

	local data = global[player.name].database[index]
	
	if (data.type == "blueprint") then
	
		local stack = getWritableBlueprint(player)
		
		if (not stack) then
			player.print({"msg_error_no_empty_blueprint"})
			return
		end
	
		stack.label = data.data.label
		stack.set_blueprint_entities(data.data.entities)
		stack.set_blueprint_tiles(data.data.tiles)
		stack.blueprint_icons = data.data.icons
		
	else
	
		local stack = getWritableBlueprintBook(player)
		
		if (not stack) then
			player.print({"msg_error_no_empty_blueprint_book"})
			return
		end
		
		stack.label = data.data.label
		stack.get_inventory(defines.inventory.item_main).clear()
		
		if data.data.main then
			
			for i, t in ipairs(data.data.main) do
			
				stack.get_inventory(defines.inventory.item_main).insert({name = "blueprint", count = 1})
				
				if data.data.main[i].label then
					stack.get_inventory(defines.inventory.item_main)[i].label = data.data.main[i].label
				end
				stack.get_inventory(defines.inventory.item_main)[i].set_blueprint_entities(data.data.main[i].entities)
				stack.get_inventory(defines.inventory.item_main)[i].set_blueprint_tiles(data.data.main[i].tiles)
				stack.get_inventory(defines.inventory.item_main)[i].blueprint_icons = data.data.main[i].icons
			
			end
		
		end
	
	end

end --}}}

function renameEntry(player) --{{{

	local string = player.gui.center["kblueprints_popup"]["kblueprints_popupInputField"].text:trim()
	local index = global[player.name].tmpIndex
	
	if (string == global[player.name].database[index].data.label) then
		player.gui.center["kblueprints_popup"].destroy()
		return false
	end
	
	if checkEntryName(player, string) then
		
		player.gui.center["kblueprints_popup"].destroy()
		global[player.name].database[index].data.label = string
		
		sortBlueprints(player)
		
		return true
	
	end
	
	return false
	
end --}}}

function repairBlueprint(player) --{{{

	local data = getDataTableFromCursor(player)
	
	if data and (data.type == "blueprint") then
		
		local offset = {-0.5, -0.5}
	
		local entities = data.data.entities
		if entities then
		
			local t = {
				["rail-signal"]			= true,
				["rail-chain-signal"]	= true,
				["straight-rail"]		= true,
				["curved-rail"]			= true,
				["train-stop"]			= true,
				["smart-train-stop"]	= true
			}
			
			for _, entity in ipairs(entities) do
				if t[entity.name] then
					offset = {-1, -1}
					break
				end
			end
			
			for _, entity in ipairs(entities) do
				entity.position.x = entity.position.x + offset[1]
				entity.position.y = entity.position.y + offset[2]
			end
			
			player.cursor_stack.set_blueprint_entities(entities)
		
		end
		
		local tiles = data.data.tiles
		if tiles then
		
			for _, tile in ipairs(tiles) do
				entity.position.x = entity.position.x + offset[1]
				entity.position.y = entity.position.y + offset[2]
			end
			
			player.cursor_stack.set_blueprint_tiles(tiles)
		
		end
		
	else
		player.print({"msg_error_empty_cursor"})
	end

end --}}}

function sortBlueprints(player) --{{{
	table.sort(global[player.name].database, function(t1, t2)
		return string.lower(t1.data.label) < string.lower(t2.data.label)
	end)
end --}}}



-- =========================================================================
-- {[> Graphical User Interface
-- ============================
function createAddDialog(player, mode, data) --{{{

	if (mode == nil) then
	
		local data = getDataTableFromCursor(player)
		
		if data and ((data.type == "blueprint") or ((data.type == "blueprint-book") and (data.data.active or data.data.main))) then
		
			if data.data.label then
			
				if (not checkEntryName(player, data.data.label)) then
					return
				end
				
				addEntry(player, data)
				return true
			end
		
			global[player.name].tmpBlueprintData = data
			return createAddDialog(player, "create")
			
		else
			global[player.name].tmpBlueprintData = nil
			return createAddDialog(player, "import")
		end
	end

	global[player.name].popup = "createOrImport"

	if (mode == "create") then
		createPopup(player, "Enter Name:", "Create", "Cancel")
	elseif (mode == "import") then
		createPopup(player, "Import String:", "Import", "Cancel")
	end
	
	return false

end --}}}

function createControlFrame(player, parent) --{{{

	local frame = parent.add({type = "frame",	name = "kblueprints_controlFrame",	style = "kblueprint_frame_style"})
	local flow = frame.add({type = "flow",		direction = "horizontal",			style = "kblueprint_flow_style_horizontal"})

	flow.add({type = "button",	name = "kblueprints_addButton",						style = "kblueprints_add_button_style"})

	flow.add({type = "label",	caption = "  ",										style = "label_style"})
	
	flow.add({type = "button",	name = "kblueprints_exportCollectionButton",		style = "kblueprints_export_button_style"})
	flow.add({type = "button",	name = "kblueprints_importCollectionButton",		style = "kblueprints_import_button_style"})

	flow.add({type = "label",	caption = "  ",										style = "label_style"})
	
	flow.add({type = "button",	name = "kblueprints_repairButton",					style = "kblueprints_repair_button_style"})
	
	flow.add({type = "label",	caption = "  ",										style = "label_style"})

	flow.add({type = "button",	name = "kblueprints_flipHorizontallyButton",		style = "kblueprints_flip_horizontally_button_style"})
	flow.add({type = "button",	name = "kblueprints_flipVerticallyButton",			style = "kblueprints_flip_vertically_button_style"})

end --}}}

function createEntryComponent(player, index, parent) --{{{

	local name = global[player.name].database[index].data.label
	local type = global[player.name].database[index].type

	local frame = parent.add({type = "frame",	name = "kblueprints_blueprintFrame_" .. index,	style = "kblueprint_frame_style"})
	local flow = frame.add({type = "flow",		direction = "horizontal",						style = "kblueprint_flow_style_horizontal"})

	flow.add({type = "button",		name = "kblueprints_DeleteEntryButton_" .. index,			style = "kblueprints_delete_button_style"})
	flow.add({type = "button",		name = "kblueprints_RenameEntryButton_" .. index,			style = "kblueprints_rename_button_style"})
	flow.add({type = "button",		name = "kblueprints_ExportEntryButton_" .. index,			style = "kblueprints_export_button_style"})
	
	if (type == "blueprint") then
		flow.add({type = "button",	name = "kblueprints_LoadEntryButton_" .. index,				style = "kblueprints_load_blueprint_button_style"})
	else
		flow.add({type = "button",	name = "kblueprints_LoadEntryButton_" .. index,				style = "kblueprints_load_blueprint_book_button_style"})
	end
	
	flow.add({type = "label",		caption = name,												style = "kblueprint_label_style"})

end --}}}

function createImportCollectionDialog(player) --{{{
	global[player.name].popup = "importCollection"
	createPopup(player, "Import String:", "Import", "Cancel")
end --}}}

function createMainButton(player) --{{{

	local gui = player.gui.top

	if (gui["kblueprints_mainButton"] ==  nil) then
		gui.add({type = "button", name = "kblueprints_mainButton", caption = "KBM", style = "kblueprint_main_button_style"})
	end

end --}}}

function createMainFrame(player) --{{{

	local gui = player.gui.left

	if gui.kblueprints_mainFlow then
		gui.kblueprints_mainFlow.destroy()
	end



	local mainFlow = gui.add({type = "flow", name = "kblueprints_mainFlow", direction = "vertical", style = "kblueprint_flow_style_vertical"})



	createControlFrame(player, mainFlow)
	
	
	
	local scrollPane = mainFlow.add({type = "scroll-pane", style = "kblueprint_scrollpane_style"})
	scrollPane.style.maximal_height = 248
	scrollPane.horizontal_scroll_policy = "never"
	scrollPane.vertical_scroll_policy = "auto"

	local viewport = scrollPane.add({type = "flow", direction = "vertical", style = "kblueprint_flow_style_vertical"})
	
	for i = 1, #(global[player.name].database) do
		createEntryComponent(player, i, viewport)
	end

end --}}}

function createPopup(player, label, proceedCaption, cancelCaption) --{{{

	local frame = player.gui.center["kblueprints_popup"]

	if frame then
		frame.destroy()
	end

	frame = player.gui.center.add({type = "frame",	name = "kblueprints_popup",	direction = "vertical",	style = "kblueprint_frame_style"})

	local flow1 = frame.add({type = "flow",	direction = "horizontal",	style = "kblueprint_flow_style_horizontal"})
	flow1.add({type = "label",					caption = label})
	frame.add({type = "textfield",				name = "kblueprints_popupInputField"})

	local flow2 = frame.add({type = "flow",	direction = "horizontal",					style = "kblueprint_flow_style_horizontal"})
	flow2.add({type = "button",					name = "kblueprints_popupProceedButton",	caption = proceedCaption,						style = "kblueprint_big_button_style"})
	flow2.add({type = "button",					name = "kblueprints_popupCancelButton", 	caption = cancelCaption,						style = "kblueprint_big_button_style"})

end --}}}

function createRenameDialog(player, index) --{{{
	global[player.name].popup = "rename"
	global[player.name].tmpIndex = index
	createPopup(player, "New name:", "Rename", "Cancel")
	player.gui.center["kblueprints_popup"]["kblueprints_popupInputField"].text = global[player.name].database[index].data.label
end --}}}

function getIndexFromButton(btnName)--{{{

	local index, tmpIndex
	local len = #(btnName)
	
	for i = 0, len do
		tmpIndex = tonumber(btnName:sub(len - i))
		if tmpIndex ~= nil then
			index = tmpIndex
		end
	end
	
	return index
	
end --}}}

function toggleGUI(player) --{{{

	local gui = player.gui.left

	if player.gui.left.kblueprints_mainFlow then
		player.gui.left.kblueprints_mainFlow.destroy()
	else
		createMainFrame(player)
	end

end --}}}



-- =========================================================================
-- {[> Remote Calls
-- ================
function resetGUI(player) --{{{

	if (player == nil) then
		player = game.player	
	end

	initializePlayer(player)
	
	if player.gui.top.kblueprints_mainButton then
		player.gui.top.kblueprints_mainButton.destroy()
	end
	
	if player.gui.left.kblueprints_mainFlow then
		player.gui.left.kblueprints_mainFlow.destroy()
	end
	
	createMainButton(player)
	
end --}}}



-- =========================================================================
-- {[> Event-Handling
-- ==================
function onGuiClick(event) --{{{

	local player = game.players[event.player_index]
	local element = event.element.name
	local refresh
	
	if (element ==  "kblueprints_mainButton") then
		toggleGUI(player)

	elseif (element ==  "kblueprints_addButton") then
		refresh = createAddDialog(player)

	elseif (element ==  "kblueprints_exportCollectionButton") then
		exportCollection(player)

	elseif (element ==  "kblueprints_importCollectionButton") then
		createImportCollectionDialog(player)

	elseif (element ==  "kblueprints_repairButton") then
		repairBlueprint(player)

	elseif (element ==  "kblueprints_flipHorizontallyButton") then
		flipHorizontally(player)

	elseif (element ==  "kblueprints_flipVerticallyButton") then
		flipVertically(player)

	elseif (element ==  "kblueprints_popupCancelButton") then
		player.gui.center["kblueprints_popup"].destroy()

	elseif (element ==  "kblueprints_popupProceedButton") then
		if (global[player.name].popup == "createOrImport") then
			refresh = createOrImportEntry(player)
		elseif (global[player.name].popup == "rename") then
			refresh = renameEntry(player)
		elseif (global[player.name].popup == "importCollection") then
			refresh = importCollection(player)
		end

	else

		local index = getIndexFromButton(element)
		
		if index then
		
			if (element:sub(13, 23) == "DeleteEntry") then
				refresh = deleteEntry(player, index)

			elseif (element:sub(13, 23) == "RenameEntry") then
				createRenameDialog(player, index)

			elseif (element:sub(13, 23) == "ExportEntry") then
				exportEntry(player, index)

			elseif (element:sub(13, 21) == "LoadEntry") then
				loadEntry(player, index)
			end
		end
	end



	if refresh then
		createMainFrame(player)
	end

end --}}}

function onPlayerJoined(event) --{{{
	initializePlayer(game.players[event.player_index])
end --}}}



-- =========================================================================
-- {[> Initialization
-- ==================  {{{

	script.on_init(function()
		for _, player in pairs(game.players) do
			initializePlayer(player)
		end
	end)
	
	script.on_configuration_changed(function(data)
	
		-- when upgrading to 1.1.x, delete old field 'blueprints' and create field 'database'
		for _, v in pairs(global) do
		
			if v.blueprints then
				
				v.database = {}
				
				for _, t in ipairs(v.blueprints) do
					
					table.insert(v.database, {
						type = "blueprint",
						version = "0.13.0",
						data = {
							label = t.name,
							entities = t.data.entities,
							tiles = t.data.tiles,
							icons = t.data.icons
						}
					})
				
				end
				
				v.blueprints = nil
				
			end
		end
		
		-- every version change, reset the gui
		for _, player in pairs(game.players) do
			resetGUI(player)
		end
	
	end)
	
	script.on_event(defines.events.on_gui_click,			onGuiClick)
	script.on_event(defines.events.on_player_joined_game,	onPlayerJoined)
	
	script.on_event("kblueprints_flip_horizontally", 		function(event) flipHorizontally(game.players[event.player_index]) end)
	script.on_event("kblueprints_flip_vertically", 			function(event) flipVertically(game.players[event.player_index]) end)
	
	remote.add_interface("KBlueprints", {
		["Reset GUI"] = resetGUI
	})

--}}}